package com.muyu.goods.domain;/**
 * @Author：尚志豪
 * @Package：com.muyu.goods.domain
 * @Project：cloud-server
 * @name：TeamProductDiscountPriceMode
 * @Date：2024/11/21 16:35
 */

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * @program: cloud-server
 * @description: 团购商品优惠力度模型
 * @author: CuiFu Bo
 * @create: 2024-11-21 16:35
 **/

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class TeamProductDiscountPriceMode {
    /**
     * 商品价格
     */
    private BigDecimal productPrice;

    /**
     * 团购优惠价格
     */
    private BigDecimal teamPrice;

    /**
     * 优惠力度 (商品价格 - 团购优惠价格) / 商品价格
     */
    private Double discount;

    public static TeamProductDiscountPriceMode of(BigDecimal productPrice, BigDecimal teamPrice){
       return TeamProductDiscountPriceMode.builder()
                .productPrice(productPrice)
                .teamPrice(teamPrice)
                .discount(
                        productPrice.subtract(teamPrice).divide(productPrice,2, RoundingMode.HALF_UP).doubleValue()
                )
                .build();

    }

}
