package com.muyu.goods.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.goods.domain.ActivityTeamInfo;
import com.muyu.goods.domain.ActivityTeamProductSkuInfo;
import com.muyu.goods.domain.Model.ActivityTeamInfoListModel;
import com.muyu.goods.domain.Model.ActivityTeamInfoListQueryModel;
import com.muyu.goods.domain.Model.TeamProductStockModel;
import com.muyu.goods.domain.TeamProductDiscountPriceMode;
import com.muyu.goods.domain.add.ActivityTeamInfoAdd;
import com.muyu.goods.remote.InvokeUserController;
import com.muyu.goods.service.ActivityTeamInfoService;
import com.muyu.goods.mapper.ActivityTeamInfoMapper;
import com.muyu.goods.service.ActivityTeamOpenInfoService;
import com.muyu.goods.service.ActivityTeamProductSkuInfoService;
import com.muyu.product.domain.model.RuleAttrAddModel;
import com.muyu.product.domain.model.RuleInfoAddModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
*
*/
@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo>
implements ActivityTeamInfoService{

    @Autowired
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;

    @Autowired
    private ActivityTeamInfoMapper activityTeamInfoMapper;

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;
    @Autowired
    private InvokeUserController invokeUserController;
    /**
     * 商品拼团信息列表
     */

    @Override
    public TableDataInfo<ActivityTeamInfoListModel> querya(ActivityTeamInfoListQueryModel activityTeamInfoListQueryModel) {

        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()),ActivityTeamInfo::getName,activityTeamInfoListQueryModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()),ActivityTeamInfo::getStatus,activityTeamInfoListQueryModel.getStatus());

        Page<ActivityTeamInfo> activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(), queryWrapper);
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                        (activityTeamInfoListModelBuilder) -> {
                            TeamProductDiscountPriceMode discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                            TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
                            Long teamOpenTypeNumber = activityTeamOpenInfoService.getTeamOpenTypeNumberByTeamId(activityTeamInfo.getId());
                            Long teamInTypeNumber = activityTeamOpenInfoService.getTeamInTypeNumberByTeamId(activityTeamInfo.getId());

                            return activityTeamInfoListModelBuilder
                                    .openTeamNumber(teamOpenTypeNumber)
                                    .addTeamNumber(teamInTypeNumber)
                                    .attendNumber(teamOpenTypeNumber+teamInTypeNumber)
                                    .endTime(activityTeamInfo.getEndTime())
                                    .productImage(activityTeamInfo.getProductImage())
                                    .teamPrice(discountPrice.getTeamPrice())
                                    .productPrice(discountPrice.getProductPrice())
                                    .teamStock(teamProductStockModel.getTeamStock())
                                    .remainStock(teamProductStockModel.getRemainStock())
                                    .status(activityTeamInfo.getStatus())
                                    .build();
                        }
                )).toList();


        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
        tableDataInfo.setRows(activityTeamInfoListModels);
        return tableDataInfo;
    }
    /**
     * 添加
     */
    @Override
    @Transactional
    public void add(ActivityTeamInfoAdd activityTeamInfoAdd) {

        // 商品拼团信息
        activityTeamInfoMapper.add(activityTeamInfoAdd);
        // 返回只id
        Long id = activityTeamInfoAdd.getId();

        // 获取list集合
        List<ActivityTeamProductSkuInfo> ruleInfoAddList = activityTeamInfoAdd.getRuleInfoAddList();

        // 循环sku添加
        for (ActivityTeamProductSkuInfo info : ruleInfoAddList) {
            System.out.println(info);
            info.setTeamId(id);
            info.setProductId(activityTeamInfoAdd.getProductId());
            activityTeamInfoMapper.addlist(info);
        }

    }


}


















