package com.muyu.goods.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.goods.domain.ActivityTeamInfo;
import com.muyu.goods.domain.ActivityTeamProductSkuInfo;
import com.muyu.goods.domain.Model.TeamProductStockModel;
import com.muyu.goods.domain.TeamProductDiscountPriceMode;
import com.muyu.goods.service.ActivityTeamProductSkuInfoService;
import com.muyu.goods.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

/**
*
*/
@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo>
implements ActivityTeamProductSkuInfoService{

    @Autowired
    private ProjectSkuCache projectSkuCache;
    @Autowired
    private ActivityTeamProductSkuInfoMapper activityTeamProductSkuInfoMapper;
    /**
     * 通过团购活动Id获取团购中最优惠的价格
     * @param teamId
     * @return
     */
    @Override
    public TeamProductDiscountPriceMode getDiscountPrice(Long teamId) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId, teamId);
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.list(lambdaQueryWrapper);

        // 优惠模型集合
       Optional<TeamProductDiscountPriceMode> discountPriceModeOptional = teamProductSkuInfoList.stream()
                .map(activityTeamProductSkuInfo -> {
            ProjectSkuInfo projectSkuInfo = projectSkuCache.get(activityTeamProductSkuInfo.getProductId(),activityTeamProductSkuInfo.getProductSku());
            return TeamProductDiscountPriceMode.of(projectSkuInfo.getPrice(),activityTeamProductSkuInfo.getTeamPrice());
        }).min(((o1, o2) -> Double.valueOf(o1.getDiscount() * 100 - o2.getDiscount() * 100).intValue()));

       if (discountPriceModeOptional.isEmpty()){
           throw new ServiceException("团购活动下没有商品绑定");
       }
        return discountPriceModeOptional.get();
    }
    /**
     * 通过活动Id获取 剩余库存
     * @param teamId
     * @return
     */
    @Override
    public TeamProductStockModel getStock(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L,Long::sum))
                .remainStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L,Long::sum))
                .build();
    }
    /**
     * 列表
     * @return
     */
    @Override
    public List<ActivityTeamProductSkuInfo> lists() {
        return activityTeamProductSkuInfoMapper.lists();
    }
}
