package com.muyu.marketing.team.controller;


import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.req.ActivityTeamInfoUpdReq;
import com.muyu.marketing.domain.req.TeamInfoListReq;
import com.muyu.marketing.domain.resp.TeamInfoFindByIdResp;
import com.muyu.marketing.domain.resp.TeamInfoListResp;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 营销团购活动控制层
 *
 * @author DongZeLiang
 * @date 2024-11-20 14:25
 */
@RestController
@RequestMapping("/team")
public class ActivityTeamController {

    @Autowired
    private ActivityTeamInfoService activityTeamInfoService;
    @Autowired
    private ActivityTeamProductSkuInfoService skuInfoService;

    /**
     * 查询营销团购活动列表
     * @param teamInfoListReq 活动查询入参
     * @return 活动响应结果
     */
    @PostMapping("/list")
    public Result<TableDataInfo<TeamInfoListResp>> list(@RequestBody TeamInfoListReq teamInfoListReq) {
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = activityTeamInfoService.query(teamInfoListReq.buildQueryModel());
        List<TeamInfoListResp> respList = tableDataInfo.getRows().stream().map(TeamInfoListResp::listModelBuild).toList();
        return Result.success(
                new TableDataInfo<>(){{
                    setRows(respList);
                    setTotal(tableDataInfo.getTotal());
                }}
        );
    }


    /**
     * 添加团购活动
     * @param activityTeamInfoSaveReq 添加请求对象
     * @return 结果集
     */
    @PostMapping("/save")
    public Result<String> save(@RequestBody ActivityTeamInfoSaveReq activityTeamInfoSaveReq) {
        activityTeamInfoService.save(ActivityTeamInfoAddModel.addReqBuild(activityTeamInfoSaveReq));
        return Result.success("添加成功");
    }

    /**
     * 根据id查询
     */
    @PostMapping("/detail")
    public List<ActivityTeamProductSkuModel> findListByTeamId(Long teamId) {
        List<ActivityTeamProductSkuModel> list = skuInfoService.findListByTeamId(teamId);
        return list;
    }

    /**
     * 评论
     * @param id
     * @return
     */
    @GetMapping("/findTeam/{id}")
    public Result<List<ActivityTeamInfo>> findTeam(@PathVariable("id") Long id){
        return Result.success(activityTeamInfoService.findTeam(id));
    }

    /** 根据id回显
     * @param id
     * @return
     */
    @PostMapping("/findById/{id}")
    public Result<TeamInfoFindByIdResp> findTeamById(@PathVariable Long id) {
        return Result.success(TeamInfoFindByIdResp.teamInfoFindByIdBuild(activityTeamInfoService.findDetailById(id)));
    }

    @PutMapping("/updateByTeamId")
    public Result updateByTeamId(@RequestBody ActivityTeamInfoUpdReq activityTeamInfoUpdReq){
        activityTeamInfoService.update(ActivityTeamInfoUpdModel.activityTeamInfoUpdReqModelBuild(activityTeamInfoUpdReq));
        return Result.success();
    }

    /**
     * 团购活动列表
     */
    @GetMapping("/teamList")
    public Result<TableDataInfo<ActivityTeamInfo>> teamList( TeamInfoListReq teamInfoListReq){
        PageHelper.startPage(teamInfoListReq.getPageNum(), teamInfoListReq.getPageSize());
        PageInfo<ActivityTeamInfo> pageInfo = new PageInfo<>(activityTeamInfoService.list());
        TableDataInfo<ActivityTeamInfo> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setRows(pageInfo.getList());
        tableDataInfo.setTotal(pageInfo.getTotal());
        return Result.success(tableDataInfo);
    }
}
