package com.muyu.product.service.impl;

import java.util.ArrayList;
import java.util.List;

import com.github.pagehelper.PageInfo;
import com.muyu.common.core.text.Convert;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.RuleAttrInfo;
import com.muyu.product.domain.model.RuleAttrAddModel;
import com.muyu.product.domain.model.RuleInfoAddModel;
import com.muyu.product.domain.req.RuleAttrInfoReq;
import com.muyu.product.domain.req.RuleInfoQueryReq;
import com.muyu.product.domain.resp.RuleInfoResp;
import com.muyu.product.mapper.RuleAttrInfoMapper;
import com.muyu.product.service.RuleAttrInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.RuleInfoMapper;
import com.muyu.product.domain.RuleInfo;
import com.muyu.product.service.RuleInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.util.CollectionUtils;

/**
 * 商品规格Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class RuleInfoServiceImpl extends ServiceImpl<RuleInfoMapper, RuleInfo>  implements RuleInfoService {

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;
    @Autowired
    private RuleAttrInfoMapper ruleAttrInfoMapper;
    @Autowired
    private ProjectInfoServiceImpl projectInfoService;

    /**
     * 查询商品规格列表
     *
     * @param ruleInfo 商品规格
     * @return 商品规格
     */
    @Override
    public List<RuleInfo> list(RuleInfo ruleInfo) {
        LambdaQueryWrapper<RuleInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(ruleInfo.getName())){
            queryWrapper.like(RuleInfo::getName, ruleInfo.getName());
        }

        if (ObjUtils.notNull(ruleInfo.getStatus())){
            queryWrapper.eq(RuleInfo::getStatus, ruleInfo.getStatus());
        }





        return list(queryWrapper);
    }

    /**
     * 添加规格信息
     *
     * @param ruleInfoAddModel 规格添加模型
     *
     * @return 时候成功
     */
    @Override
    public boolean save (RuleInfoAddModel ruleInfoAddModel) {
        RuleInfo ruleInfo = RuleInfo.addModelBuild(ruleInfoAddModel, SecurityUtils::getUsername);
        boolean save = this.save(ruleInfo);
        ruleAttrInfoService.saveBatch(
                ruleInfoAddModel.getRuleAttrList().stream()
                        .map(ruleAttrAddModel -> RuleAttrInfo.addModelBuild(ruleAttrAddModel, ruleInfo::getId, SecurityUtils::getUsername))
                        .toList()
        );
        return save;
    }

    /**
     * 查询规格信息
     * @param ruleInfoQueryReq 规格查询
     * @return 规格信息
     */
    @Override
    public TableDataInfo<RuleInfoResp> queryList (RuleInfoQueryReq ruleInfoQueryReq) {
        List<RuleInfo> list = this.list(RuleInfo.queryBuild(ruleInfoQueryReq));
        List<RuleInfoResp> ruleInfoRespList = list.stream()
                .map(ruleInfo -> RuleInfoResp.infoBuild(ruleInfo, ruleId -> {
                        LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
                        queryWrapper.eq(RuleAttrInfo::getRuleId, ruleId);
                        return ruleAttrInfoService.list(queryWrapper).stream().map(RuleAttrAddModel::infoBuild).toList();
                    }))
                .toList();
        boolean isPage = ruleInfoQueryReq.getParams().get("isPage") == null || Convert.toBool(ruleInfoQueryReq.getParams().get("isPage"), true);
              ruleInfoRespList.forEach(ruleInfoResp -> {
                  LambdaQueryWrapper<ProjectInfo> wrapper = new LambdaQueryWrapper<>();
                  wrapper.eq(ProjectInfo::getRuleId,ruleInfoResp.getId());
                  List<ProjectInfo> projectInfoList = projectInfoService.list(wrapper);
                  if (!CollectionUtils.isEmpty(projectInfoList) && projectInfoList.size()>0){
                      ruleInfoResp.setState(1);
                  }
              });
        return TableDataInfo.<RuleInfoResp>builder()
                .rows(ruleInfoRespList)
                .total(isPage ? new PageInfo<>(list).getTotal() : 0)
                .build();
    }
    /**
     * 获取商品规格详细信息
     */
    @Override
    public RuleInfoAddModel getRuleAttrInfoResp(Long id) {
        //获取商品规格
        RuleInfo byId = this.getById(id);
        if (byId==null){
            throw new RuntimeException("没有这个商品规格");
        }
        List<RuleAttrInfo> ruleAttrInfos=ruleAttrInfoService.getRuleAttrInfo(byId.getId());
        ArrayList<RuleAttrAddModel> arrayList = new ArrayList<>();
        RuleAttrAddModel ruleAttrAddModel = new RuleAttrAddModel();
        for (RuleAttrInfo ruleAttrInfo : ruleAttrInfos) {
            ruleAttrAddModel.setName(ruleAttrInfo.getName());
        }
        List<String> list = ruleAttrInfos.stream().map(RuleAttrInfo::getAttrValue).toList();
        ruleAttrAddModel.setValueList(list);
        arrayList.add(ruleAttrAddModel);

        RuleInfoAddModel resp = new RuleInfoAddModel();
        resp.setId(byId.getId());
        resp.setName(byId.getName());
        resp.setStatus(byId.getStatus());
        resp.setRuleAttrList(arrayList);
        return resp;
    }
    /**
     * 修改商品规格
     */
    @Override
    public void updateRuleById(Long id, RuleAttrInfoReq req) {
        //修改商品规格表
        ruleAttrInfoMapper.updRule(id,req);
        //删除规格详情表
        ruleAttrInfoMapper.deleteRule(id);
        //添加规格详情表
        List<RuleAttrAddModel> ruleAttrList = req.getRuleAttrList();
        for (RuleAttrAddModel ruleAttrAddModel : ruleAttrList) {
            List<String> valueList = ruleAttrAddModel.getValueList();
            String value="";
            String valueData=null;
            for (String s : valueList) {
                valueData = value + "," + s;
            }
            String substring = valueData.substring(1);
            System.out.println(substring);
            ruleAttrInfoMapper.addRuleAttrModel(id,ruleAttrAddModel.getName(),substring);
        }
    }
    /**
     * 修改状态
     */
    @Override
    public int changeTheStatusById(Long id, String status) {
        RuleInfo ruleInfo = RuleInfo.editUpdBuild(id,status);
        this.updateById(ruleInfo);
        return 1;
    }


}
